<?
/*
 Network Interface Statistics Collection Agent v2.5
 by Brett Baugh

 This is where the magic happens, for this is the report
 generation script. If it's weird and funky, you'll find
 it in here somewhere; every other script is vanilla in
 comparison. It weaves a web of mathematical voodoo to
 make the graphs and print the tables. I've tried to
 comment it fairly well (meaning, "more than none at all")
 so you can hack at it if you want. A bit. Good luck.

 Suffice to say, this page is the entire reason NISCA exists.
 */

if (isset($load_report)) {
  header ("Location: index.php?load_report=$report");
  die;
}

set_time_limit(0);
require_once("/etc/nisca/nisca.conf");
require_once($location["functions"]);
$random_msgs="";

// Clean up after yourself... delete all JPGs and PNGs older than one minute, that is.
$d=dir(".");
while ($f=$d->read()) {
  if (preg_match('/^(bytes|packets|errors|dropped)_\d+\.(jpg|png)/i', $f)
      && filemtime($f) < (time()-60)) {
    unlink($f);
  }
}
$d->close();

// normalize() changes the values in $arr from running totals to subtotals per stamp.
// It also will set $arr[0] to zero; just part of the flooring process.
function normalize($arr) {
  global $cnt_stamps;
  $tmp_stamps=$cnt_stamps;
  $floor=$arr[$tmp_stamps[0]];
  $out=array();
  $pstamp=0;
  for ($i=0; $i<count($arr); $i++) {
    $stamp=$tmp_stamps[$i];
    $out[$stamp] = $arr[$stamp]-$floor;
    if ($out[$stamp] < 0) {
      $out[$stamp] = $out[$pstamp];
    }
    $floor=$arr[$stamp];
    $pstamp=$stamp;
  }
  return $out;
}

function do95(&$rarray, &$tarray) {
  global $do_in, $do_out;
  if ($do_in) {
    $tmp95=$rarray;
    arsort($tmp95, SORT_NUMERIC);
    $toss_cnt=floor(count($tmp95)*0.05);
    $i=0;
    while(list($elem,)=each($tmp95)) {
      unset($rarray[$elem]);
      if (++$i>=$toss_cnt) { break; }
    }
  }
  unset($tmp95);
  if ($do_out) {
    $tmp95=$tarray;
    arsort($tmp95, SORT_NUMERIC);
    $toss_cnt=floor(count($tmp95)*0.05);
    $i=0;
    while(list($elem,)=each($tmp95)) {
      unset($tarray[$elem]);
      if (++$i>=$toss_cnt) { break; }
    }
  }
  unset($tmp95);
  return TRUE;
}

function makemod($val, $more_mod) {
  $modifier=1;
  $mod_disp="";
  if ($val >= 1000000000/$more_mod) {
    $modifier = 1000000000;
    $mod_disp="G";
  }
  elseif ($val >= 1000000/$more_mod) {
    $modifier = 1000000;
    $mod_disp="M";
  }
  elseif ($val >= 1000/$more_mod) {
    $modifier = 1000;
    $mod_disp="K";
  }
  return array($modifier, $mod_disp);
}

/*
Give makegraph() the incoming and/or outgoing $cnt_ arrays and it'll return an image pointer
to the graph it creates. It does NOT determine whether it'll be a jpeg or png or whatnot.

To speed this up, I tried limiting the datapoints depending on the number
of pixels available to graph on... but it doesn't work, because it still
has to process *all* the database entries to get accurate averages. Adding
the "speeding-up" algorithm actually makes it slower.  Sorry... :(

Anyone have any ideas on how to make this faster when you have 100,000
entries to chew through? My machine still takes 10 seconds to do that many
(after the mysql results are cached, that is; 40 seconds before) and it's a 1.2 GHz...
 */

function makegraph($rarray, $tarray, $gtype, $cir) {
  global $X, $Y, $cnt_stamps, $ttf_font, $bitbyte, $in_out,
    $graph_background, $graph_text, $graph_border_light, $graph_border_dark,
    $graph_peak_highlight, $graph_incoming_data, $graph_outgoing_data,
    $dateformat, $dformat, $tformat, $flubber, $map, $do95;
  $img=imagecreate($X, $Y);

  if (eregi("i", $in_out)) { $do_in="y"; } else { unset($do_in); }
  if (eregi("o", $in_out)) { $do_out="y"; } else { unset($do_out); }

// REMEMBER: whichever color is allocated first becomes the background color!
  $r = hexdec(substr($graph_background, 0, 2));
  $g = hexdec(substr($graph_background, 2, 2));
  $b = hexdec(substr($graph_background, 4, 2));
  $BGCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_text, 0, 2));
  $g = hexdec(substr($graph_text, 2, 2));
  $b = hexdec(substr($graph_text, 4, 2));
  $TEXTCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_border_light, 0, 2));
  $g = hexdec(substr($graph_border_light, 2, 2));
  $b = hexdec(substr($graph_border_light, 4, 2));
  $LBORDERCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_border_dark, 0, 2));
  $g = hexdec(substr($graph_border_dark, 2, 2));
  $b = hexdec(substr($graph_border_dark, 4, 2));
  $DBORDERCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_peak_highlight, 0, 2));
  $g = hexdec(substr($graph_peak_highlight, 2, 2));
  $b = hexdec(substr($graph_peak_highlight, 4, 2));
  $PEAKCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_incoming_data, 0, 2));
  $g = hexdec(substr($graph_incoming_data, 2, 2));
  $b = hexdec(substr($graph_incoming_data, 4, 2));
  $INCOLOR=imagecolorallocate($img, $r, $g, $b);
  $r = hexdec(substr($graph_outgoing_data, 0, 2));
  $g = hexdec(substr($graph_outgoing_data, 2, 2));
  $b = hexdec(substr($graph_outgoing_data, 4, 2));
  $OUTCOLOR=imagecolorallocate($img, $r, $g, $b);

// I HATE interlaced graphics.  grrrr.
  imageinterlace($img, 0);

// Draw the outer border
  imageline ($img, 0, 0, $X - 1, 0, $LBORDERCOLOR);
  imageline ($img, 1, 1, $X - 2, 1, $LBORDERCOLOR);
  imageline ($img, 0, 0, 0, $Y - 1, $LBORDERCOLOR);
  imageline ($img, 1, 1, 1, $Y - 2, $LBORDERCOLOR);
  imageline ($img, $X - 1, 0, $X-1, $Y - 1, $DBORDERCOLOR);
  imageline ($img, 0, $Y - 1, $X-1, $Y - 1, $DBORDERCOLOR);
  imageline ($img, $X - 2, 1, $X-2, $Y - 2, $DBORDERCOLOR);
  imageline ($img, 1, $Y - 2, $X-2, $Y - 2, $DBORDERCOLOR);

  if (count($rarray) < 3 && count($tarray) < 3) {
    $xcent=$X/2;
    $ycent=$Y/2;
    $line1="Not enough data to create graph.";
    $line2="You must have at least three data points.";
    $line3="Sorry!";

    $x1=$X-((imagefontwidth(5) * strlen($line1))/2)-($X/2);
    $x2=$X-((imagefontwidth(4) * strlen($line2))/2)-($X/2);
    $x3=$X-((imagefontwidth(4) * strlen($line3))/2)-($X/2);;
    $y1=floor($Y/4);
    $y2=$y1*2;
    $y3=$y1*3;

    imagestring ($img, 5, $x1, $y1, $line1, $PEAKCOLOR);
    imagestring ($img, 4, $x2, $y2, $line2, $TEXTCOLOR);
    imagestring ($img, 4, $x3, $y3, $line3, $TEXTCOLOR);
    return $img;
  }

// These are the inner border coordinates for the graph itself minus margins.
  $x0 = 55; $x1 = $X - 45;
  $y0 = 35; $y1 = $Y - 45;
  $num_xpixels = $x1 - $x0;
  $num_ypixels = $y1 - $y0;

  $tmp_stamps=$cnt_stamps;
// Find the smallest and largest values from the timestamps (X-axis) and both input arrays (Y-axis).
  $stamp_min=$tmp_stamps[0];
  $stamp_max=$tmp_stamps[count($tmp_stamps)-1];
  $rdata_max=$tdata_max=0;

  if ($do95) {
    if ($do_in) {
      $tmp95=$rarray;
      $toss95r=array();
      arsort($tmp95, SORT_NUMERIC);
      $toss_cnt=floor(count($tmp95)*0.05);
      $i=0;
      while(list($elem,)=each($tmp95)) {
        $toss95r[$elem]=TRUE;
        if (++$i>=$toss_cnt) { break; }
      }
    }
    unset($tmp95);
    if ($do_out) {
      $tmp95=$tarray;
      $toss95t=array();
      arsort($tmp95, SORT_NUMERIC);
      $toss_cnt=floor(count($tmp95)*0.05);
      $i=0;
      while(list($elem,)=each($tmp95)) {
        $toss95t[$elem]=TRUE;
        if (++$i>=$toss_cnt) { break; }
      }
    }
    unset($tmp95);
  }

  if ($do_in) {
    $prev_val=0;
    foreach(array_keys($rarray) as $k) {
      if ($k==$stamp_min) { continue; }
      if ($toss95r[$k]) { $rarray[$k]=$prev_value; }
      if ($rarray[$k] > $rdata_max) { $rdata_max=$rarray[$k]; }
      $prev_value=$rarray[$k];
    }
    if (! $do_out) { $data_max=$rdata_max; }
  }
  if ($do_out) {
    $prev_val=0;
    foreach(array_keys($tarray) as $k) {
      if ($k==$stamp_min) { continue; }
      if ($toss95t[$k]) { $tarray[$k]=$prev_value; }
      if ($tarray[$k] > $tdata_max) { $tdata_max=$tarray[$k]; }
      $prev_value=$tarray[$k];
    }
    if (! $do_in) { $data_max=$tdata_max; }
  }
  if ($do_in && $do_out) {
    $data_max=max($rdata_max, $tdata_max);
  }

  $more_mod=1;
  if ($bitbyte=="b" && ($gtype=="bits" || $gtype=="bytes")) { $more_mod=8; }
  list($modifier, $mod_disp) = makemod($data_max, $more_mod);
  if ($more_mod==8) { $modifier /= 8; }

// Preventing division by zero should be the dream of all programmers.
// After all... what if all 37,241 database entries are "0"?
  if ($data_max > 0) {
    $yceil = ceil($data_max / $modifier);
  } else {
    $yceil = 10;
  }
  $ystep = $num_ypixels / $yceil;
  $ygrid = $num_ypixels / 10;

// $numpoints is the number of X-axis datapoints.
  $numpoints = count($tmp_stamps);

// Draw the data color key
  if ($do_in) {
    imageline ($img, $x0, 10, $x0+40, 10, $INCOLOR);
  }
  if ($do_out) {
    imageline ($img, $x0, 23, $x0+40, 23, $OUTCOLOR);
  }
  if ($ttf_font == "") {
    if ($do_in) {
      imagestring ($img, 2, $x0+50, 3, "Incoming", $INCOLOR);
      $xrend=imagefontwidth(2) * strlen("Incoming") + $x0 + 56;
    }
    if ($do_out) {
      imagestring ($img, 2, $x0+50, 16, "Outgoing", $OUTCOLOR);
      $xtend=imagefontwidth(2) * strlen("Outgoing") + $x0 + 56;
    }
  } else {
    if ($do_in) {
      imagettftext ($img, 10, 0, $x0+50, 13, $INCOLOR, $ttf_font, "Incoming");
      $rbox=imagettfbbox(10, 0, $ttf_font, "Incoming");
      $xrend=abs($rbox[2]-$rbox[0]) + $x0 + 56;
    }
    if ($do_out) {
      imagettftext ($img, 10, 0, $x0+50, 26, $OUTCOLOR, $ttf_font, "Outgoing");
      $tbox=imagettfbbox(10, 0, $ttf_font, "Outgoing");
      $xtend=abs($tbox[2]-$tbox[0]) + $x0 + 56;
    }
  }

  $use_style=FALSE;
  if (function_exists("imagesetstyle")) {
// Create the Special Image Style Array needed in libgd2.x to make dashed lines. Ridiculous...
    $dash_style = array ($LBORDERCOLOR,$LBORDERCOLOR,$LBORDERCOLOR,$LBORDERCOLOR,
                         $BGCOLOR,$BGCOLOR,$BGCOLOR,$BGCOLOR);
    imagesetstyle($img, $dash_style);
    $use_style=TRUE;
  }

// Draw the Y-axis gridlines and labels:
  $j=0;
  for ($i=0; $i<=$yceil+.001; $i+=($yceil/10)) {
    $label = number_format(sprintf("%0.1f", $i), 1) . $mod_disp;
    if ($ttf_font == "") {
      imagestring ($img, 2, $x0 - 40, $y1 - $j * $ygrid - 7, $label, $TEXTCOLOR);
    } else {
      imagettftext ($img, 9, 0, $x0 - 44, $y1 - $j * $ygrid +3, $TEXTCOLOR, $ttf_font, $label);
    }
    if ($use_style) {
      imageline ($img, $x0, $y1 - $j * $ygrid, $x1, $y1 - $j * $ygrid, IMG_COLOR_STYLED);
    } else {
      imagedashedline ($img, $x0, $y1 - $j * $ygrid, $x1, $y1 - $j * $ygrid, $LBORDERCOLOR);
    }
    $j++;
  }

// Draw the X-axis gridlines and labels:
  $map="<map NAME=\"%%%MAPNAME%%%\">\n";
  $j=0;
  $xgrid = $num_xpixels / 11;
  for ($i=$stamp_min; $i<=$stamp_max; $i += floor(($stamp_max - $stamp_min) / 11)) {
    $pix = $j * $xgrid;
    $datelabel1=date($dformat, $i);
    $datelabel2=date($tformat, $i);
    if ($ttf_font == "") {
      imagestring($img, 2, $x0 + $pix - 22, $y1 + 8, $datelabel1, $TEXTCOLOR);
      imagestring($img, 2, $x0 + $pix - 22, $y1 + 22, $datelabel2, $TEXTCOLOR);
    } else {
      imagettftext ($img, 9, 0, $x0 + $pix - 26, $y1 + 16, $TEXTCOLOR, $ttf_font, $datelabel1);
      imagettftext ($img, 9, 0, $x0 + $pix - 24, $y1 + 31, $TEXTCOLOR, $ttf_font, $datelabel2);
    }
    if ($use_style) {
      imageline ($img, $x0 + $pix, $y0, $x0 + $pix, $y1, IMG_COLOR_STYLED);
    } else {
      imagedashedline ($img, $x0 + $pix, $y0, $x0 + $pix, $y1, $LBORDERCOLOR);
    }
// Do the flubber (clickable image map for zooming in).
    if ($prev_i) {
      $fx0=(integer)($x0+($xgrid * ($j-1)));
      $fy0=$y0;
      $fx1=(integer)($x0+$pix);
      $fy1=$y1+7;
      list($flubber["f_month"], $flubber["f_day"], $flubber["f_year"], $flubber["f_hour"],
           $flubber["f_min"], $flubber["f_sec"])=preg_split('/:/', date("n:j:Y:G:i:s", $prev_i));
      list($flubber["t_month"], $flubber["t_day"], $flubber["t_year"], $flubber["t_hour"],
           $flubber["t_min"], $flubber["t_sec"])=preg_split('/:/', date("n:j:Y:G:i:s", $i));
      $querystring="report.php?flubber=" . rawurlencode(serialize($flubber));
      $map .= "  <area SHAPE=\"rect\" COORDS=\"$fx0,$fy0,$fx1,$fy1\" HREF=\"$querystring\">\n";
    }
    $prev_i=$i;
    $j++;
  }
  $map .= "</map>\n";

  $timerange=$stamp_max-$stamp_min;
  $xpct = $num_xpixels / $timerange;
  $ypct = $yceil * $modifier;

// Draw the CIR line across it, if it's not zero and falls inside the Y-axis data range.
  if ($cir != 0 && ($cir/$modifier) <= $yceil) {
    $pointy = $y1 - ($cir * $num_ypixels) / $ypct;
    imageline($img, $x0, $pointy, $x1, $pointy, $PEAKCOLOR);
    $dlabel1=sprintf("%0.1f", $cir/$modifier) . " " . $mod_disp;
    $dlabel2="(CIR)";
    if ($ttf_font == "") {
      $yadd=imagefontheight(3)/2;
      imagestring($img, 3, $x1+5, $pointy-$yadd, $dlabel1, $TEXTCOLOR);
      imagestring($img, 3, $x1+5, $pointy+8, $dlabel2, $TEXTCOLOR);
    } else {
      $box=imagettfbbox(10, 0, $ttf_font, $dlabel1 . "\n\r" . $dlabel2);
      $yadd=abs($box[1]-$box[6])/2;
      imagettftext ($img, 10, 0, $x1+5, $pointy+($yadd/2), $TEXTCOLOR, $ttf_font, $dlabel1);
      imagettftext ($img, 10, 0, $x1+5, $pointy+($yadd*2), $TEXTCOLOR, $ttf_font, $dlabel2);
    }
  }

// Draw the inner border
  imageline ($img, $x0, $y0, $x1, $y0, $LBORDERCOLOR);
  imageline ($img, $x0, $y0, $x0, $y1, $LBORDERCOLOR);
  imageline ($img, $x1, $y1, $x0, $y1, $LBORDERCOLOR);
  imageline ($img, $x1, $y1, $x1, $y0, $LBORDERCOLOR);

// Draw the datapoint connection lines and peak labels. This is what takes forever.
  if ($do_in) {
    $prev_pointyr=$y1 - ($rarray[$tmp_stamps[1]] * $num_ypixels) / $ypct;
  }
  if ($do_out) {
    $prev_pointyt=$y1 - ($tarray[$tmp_stamps[1]] * $num_ypixels) / $ypct;
  }
  $prev_pointx=$x0;
  for ($i = 1; $i < count($tmp_stamps); $i++) {
    $stamp=$tmp_stamps[$i];
    $pointx = $x1 - ($xpct * ($stamp_max-$stamp));
    if ($do_in) {
      $pointyr = $y1 - ($rarray[$stamp] * $num_ypixels) / $ypct;
      imageline($img, $prev_pointx, $prev_pointyr, $pointx, $pointyr, $INCOLOR);
      if ($rdata_max == $rarray[$stamp] && $rdata_max != 0) {
        $rdata_max=0;
        $tlabel=date($dateformat, $stamp);
        imagearc($img, $pointx, $pointyr, 10, 10, 0, 360, $PEAKCOLOR);
        list($peak_mod, $peak_disp) = makemod($rarray[$stamp], $more_mod);
        if ($more_mod==8) { $peak_mod /= 8; }
        if ($do95) { $aboo=" (95%)"; } else { $aboo=""; }
        $rdlabel="- Peak$aboo = " . sprintf("%0.2f", $rarray[$stamp]/$peak_mod) .
                 " $peak_disp$gtype/sec";
        $rdlabel .= " at $tlabel";
        $r_xpos=$xrend;
        $r_ypos=13;
      }
      $prev_pointyr=$pointyr;
    }
    if ($do_out) {
      $pointyt = $y1 - ($tarray[$stamp] * $num_ypixels) / $ypct;
      imageline($img, $prev_pointx, $prev_pointyt, $pointx, $pointyt, $OUTCOLOR);
      if ($tdata_max == $tarray[$stamp] && $tdata_max != 0) {
        $tdata_max=0;
        $tlabel=date($dateformat, $stamp);
        imagearc($img, $pointx, $pointyt, 10, 10, 0, 360, $PEAKCOLOR);
        list($peak_mod, $peak_disp) = makemod($tarray[$stamp], $more_mod);
        if ($more_mod==8) { $peak_mod /= 8; }
        if ($do95) { $aboo=" (95%)"; } else { $aboo=""; }
        $tdlabel="- Peak$aboo = " . sprintf("%0.2f", $tarray[$stamp]/$peak_mod) .
                 " $peak_disp$gtype/sec";
        $tdlabel .= " at $tlabel";
        $t_xpos=$xtend;
        $t_ypos=26;
      }
      $prev_pointyt=$pointyt;
    }
    $prev_pointx=$pointx;
  }
// Add the most-recent speed found.
  $stamp=$tmp_stamps[$i-1];
  if ($do_in) {
    list($peak_mod, $peak_disp) = makemod($rarray[$stamp], $more_mod);
    if ($more_mod==8) { $peak_mod /= 8; }
    $rdlabel .= " - Current = " . sprintf("%0.2f", $rarray[$stamp]/$peak_mod) .
                " $peak_disp$gtype/sec";
    if (trim($ttf_font) == "") {
      $r_ypos -= 10;
      imagestring($img, 2, $r_xpos, $r_ypos, $rdlabel, $INCOLOR);
    } else {
      imagettftext ($img, 10, 0, $r_xpos, $r_ypos, $INCOLOR, $ttf_font, $rdlabel);
    }
  }
  if ($do_out) {
    list($peak_mod, $peak_disp) = makemod($tarray[$stamp], $more_mod);
    if ($more_mod==8) { $peak_mod /= 8; }
    $tdlabel .= " - Current = " . sprintf("%0.2f", $tarray[$stamp]/$peak_mod) .
                " $peak_disp$gtype/sec";
    if (trim($ttf_font) == "") {
      $t_ypos -= 10;
      imagestring($img, 2, $t_xpos, $t_ypos, $tdlabel, $OUTCOLOR);
    } else {
      imagettftext ($img, 10, 0, $t_xpos, $t_ypos, $OUTCOLOR, $ttf_font, $tdlabel);
    }
  }

  unset($tmp_stamps,$tmp_tmp_stamps,$tmp_rarray,$tmp_tarray,$rarray,$tarray);
  if (function_exists("ImageCopyMerge")) {
    return icm($img);
  } else {
    return add_watermark($img);
  }
}

$errtxt="";

if ($flubber) {
// it will contain f_month, etc.
  $tmp=unserialize(ss(rawurldecode($flubber)));
  $flubber=$tmp;
  foreach(array_keys($flubber) as $k) {
    $$k=$flubber[$k]; // heheh... tricky.
  }
}

if ($report == "" && count($hostif) < 1) {
  $errtxt .= "You have to pick at least one hostname/interface to do a report on.<BR>\n";
}

if ($report == "" && $bytes=="" && $packets=="" && $errors=="" && $dropped=="") {
  $errtxt .= "You must select one of the four report types or there'll be nothing to " .
    "report on (bytes, packets, errors, or dropped).<BR>\n";
}

if ($report == "" && ($summ_interval == "" || preg_match('/^\s*[^\d\.]+\s*$/', $summ_interval))) {
  $errtxt .= "The summary interval cannot be blank and cannot have anything but a " .
    "number in it.<BR>\n";
}

/*
Variables from index form:
$hostif[] (hostname and interface to report on)
  multiple select box
  array with each element formatted like "hostname.host.com!if_name";
  should be changed to just a list of if_id's by nisca 3.
$bytes (show byte reports if not blank)
$packets (show packet reports if not blank)
$errors (show error reports if not blank)
$dropped (show dropped packet reports if not blank)
$reportctl (select box)
  if "g", only show graphs
  if "r", only show reports
  if "gr", show both
$in_out (select box)
  if "i", show only incoming data
  if "o", show only outgoing data
  if "io", show both
$g_fmt (select box)
  if "j", make graphs as JPGs
  if "p", make graphs as PNGs
  Default to $graph_fmt (global config option)
$g_bb (select box)
  if "", use bits on graphs and reports
  if "g", use bytes on graphs, bits on reports
  if "r", use bits on graphs, bytes on reports
  if "gr", use bytes on graphs and reports
$do95 (select box)
  if 0, don't perform 95% limiting
  if 1, do
$multiple (make combined-interfaces report if not blank)
$avginterval (the number of seconds each "slot" in a multi-interface graph should be)
$timeperiod (radio button)
  if "set_dates", use date restrictions, below
  if "last_minute", restrict date to the past $time_x minutes
  if "last_hour", restrict date to the past $time_x hours
  if "last_day", restrict date to the past $time_x days
  if "", show every entry there is.
$time_x (number modifier for $timeperiod; can't be blank)
$f_month, $f_day, $f_year, $f_hour, $f_min, $f_sec (date/time of earliest DB record to report on)
$t_month, $t_day, $t_year, $t_hour, $t_min, $t_sec (date/time of latest DB record to report on)
$graph_x, $graph_y (size to make each graph, in pixels)
$summ_interval (number of $summ_units units between each summary report line)
$summ_units (units to apply to $summ_interval: "second", "minute", "hour", "month", "day", "year")
$report (load saved report "$report" if not blank)
$rpt_name (save form values as the report "$rpt_name" if it doesn't already exist)
$g_bg (hex color to use for graph background)
$g_text (hex color to use for graph text)
$g_lborder (hex color to use for the light border)
$g_dborder (hex color to use for the dark border)
$g_peak (hex color to use for the peak highlight)
$g_in (hex color to use for graphing incoming data)
$g_out (hex color to use for graphing outgoing data)

Rather a lot, isn't it?  :)
*/

if ($g_bg) { $graph_background=strtoupper($g_bg); }
if ($g_text) { $graph_text=strtoupper($g_text); }
if ($g_lborder) { $graph_border_light=strtoupper($g_lborder); }
if ($g_dborder) { $graph_border_dark=strtoupper($g_dborder); }
if ($g_peak) { $graph_peak_highlight=strtoupper($g_peak); }
if ($g_in) { $graph_incoming_data=strtoupper($g_in); }
if ($g_out) { $graph_outgoing_data=strtoupper($g_out); }

if ($rpt_name != "" && $report == "") {
  $sq="select id from reports where name='$rpt_name'";
  $res=dosql($sq);
  $hostifsql=join('*', $hostif);
  if (mysql_num_rows($res) > 0) {
    $rpt_id=mysql_result($res,0,"id");
    $random_msgs .= "The report \"$rpt_name\" exists; report NOT saved.<BR>" .
      "You must use the <a href=\"{$location["admin"]}/adm_rpt2.php?report=$rpt_id\"><font color=#$graph_text>administration section</font></a> to change existing reports.<BR>";
  } else {
    $sq="replace into reports values(0, '$rpt_name', '$hostifsql', " .
      "'$bytes', '$packets', '$errors', '$dropped', '$reportctl', '$in_out', " .
      "'$timeperiod', $time_x, $f_month, $f_day, $f_year, $f_hour, $f_min, $f_sec, " .
      "$t_month, $t_day, $t_year, $t_hour, $t_min, $t_sec, $graph_x, $graph_y, " .
      "$summ_interval, '$summ_units', '$graph_background', '$graph_text', " .
      "'$graph_border_light', '$graph_border_dark', '$graph_peak_highlight', " .
      "'$graph_outgoing_data', '$graph_incoming_data', '$g_fmt', '$multiple', $avginterval, " .
      "'$g_bb', $do95)";
    $res=dosql($sq);
    $random_msgs .= "Added the report \"$rpt_name\".<BR>";
  }
}

if ($report != "") {
  $sq="select * from reports where id=$report";
  $res=dosql($sq);
  if (mysql_num_rows($res) > 0) {
    list($crap, $rname, $hi, $bytes, $packets, $errors, $dropped, $reportctl,
        $in_out, $timeperiod, $time_x, $f_month, $f_day, $f_year, $f_hour, $f_min,
        $f_sec, $t_month, $t_day, $t_year, $t_hour, $t_min, $t_sec, $graph_x,
        $graph_y, $summ_interval, $summ_units, $graph_background, $graph_text,
        $graph_border_light, $graph_border_dark, $graph_peak_highlight,
        $graph_incoming_data, $graph_outgoing_data, $g_fmt, $multiple,
        $avginterval, $g_bb, $do95)=mysql_fetch_row($res);
    if (! $hostif=preg_split('/\*/', ss($hi))) {
      $hostif=array(ss($hi));
    }
    $random_msgs .= "Generated from saved report \"$rname\".<BR>";
  } else {
    $errtxt .= "Report number $report does not exist in the database.<BR>\n";
  }
}

if ($errtxt != "") {
?>
<html><head><title>NISCA Error</title>
<style type="text/css"> 
BODY, TD, TH { font-family: <? echo $fontface; ?>; }
</style>
</head>
<? echo default_body() . titlebar(); ?>
<hr size=1 noshade>
<? print_error($errtxt); ?>
<a href="Javascript: history.back();">Back To The Menu</a>
<noscript>
<a href="index.php">
</noscript>
</body></html>
<?
  die;
}

// we have to make this *after* the report options are loaded, if any.
// f_month, etc will be added by makegraph().
// $timeperiod will always be "set_dates".
$flubber=array("hostif"=>$hostif, "bytes"=>$bytes, "packets"=>$packets, "errors"=>$errors,
  "dropped"=>$dropped, "reportctl"=>$reportctl, "in_out"=>$in_out, "g_fmt"=>$g_fmt,
  "multiple"=>$multiple, "avginterval"=>$avginterval, "timeperiod"=>"set_dates",
  "graph_x"=>$graph_x, "graph_y"=>$graph_y, "summ_interval"=>$summ_interval,
  "summ_units"=>$summ_units, "graph_background"=>$graph_background, "graph_text"=>$graph_text,
  "graph_border_light"=>$graph_border_light, "graph_border_dark"=>$graph_border_dark,
  "graph_peak_highlight"=>$graph_peak_highlight, "graph_incoming_data"=>$graph_incoming_data,
  "graph_outgoing_data"=>$graph_outgoing_data, "g_bb"=>$g_bb, "do95"=>$do95);

// Now let's do some report generatin'...
$start_stopwatch=getmicrotime();

switch ($summ_units) {
case "minute":
  $summ_interval *= 60;
  break;
case "hour":
  $summ_interval *= 3600;
  break;
case "day":
  $summ_interval *= 86400;
  break;
case "month": // note: we'll just assume exactly 30-day months...
  $summ_interval *= 2592000;
  break;
case "year": // note: we'll just assume exactly 365-day years...
  $summ_interval *= 31536000;
  break;
}

// Set the $X and $Y graph sizes and image type.
$X = $graph_x;
$Y = $graph_y;
if ($g_fmt=="") { $g_fmt=$graph_fmt; }

// $HR is a fake <hr> because <hr color=#xxxxxx> doesn't work in netscape.
$HR="<table border=0 width=100% cellpadding=0 cellspacing=0>" .
    "<tr><td bgcolor=#$graph_text><img src=\"1x1.gif\" height=1></td></tr>" .
    "</table>\n";

?>
<html><head>
<style type="text/css">
* { font-family: <? echo $fontface; ?>; } 
.small { font-size: 11pt; }
A { text-decoration: none; }
A:hover, A:focus { text-decoration: underline; }
</style>
<title>NISCA Report</title>
</head>
<body bgcolor=#<? echo $graph_background; ?> text=#<? echo $graph_text; ?> link=#<? echo $graph_incoming_data; ?> vlink=#<? echo $graph_incoming_data; ?>  alink=#<? echo $graph_peak_highlight; ?>>
<?
echo titlebar();
if ($random_msgs != "") {
  echo "$HR\n<P><font color=#$graph_peak_highlight>$random_msgs</font></P>\n";
  $random_msgs="";
}

if (eregi("i", $in_out)) { $do_in="y"; } else { unset($do_in); }
if (eregi("o", $in_out)) { $do_out="y"; } else { unset($do_out); }

// Now we need to create our SQL query's date stamp filter if needed...
// We only need to do this once, not once for each interface.
$sql_date="";
$date_start = 0;
$date_end = time();
if ($timeperiod=="set_dates") {
  $date_start = mktime($f_hour, $f_min, $f_sec, $f_month, $f_day, $f_year);
  $date_end   = mktime($t_hour, $t_min, $t_sec, $t_month, $t_day, $t_year);
  $sql_date = "AND stamp >= $date_start AND stamp <= $date_end";
} elseif ($timeperiod=="last_minute") {
// Don't ask why I add one here; I'm sure it's very technical... :)
  $date_start = $date_end - (60 * ($time_x + 1));
  $sql_date = "AND stamp >= $date_start AND stamp <= $date_end";
} elseif ($timeperiod=="last_hour") {
  $date_start = $date_end - (3600 * $time_x);
  $sql_date = "AND stamp >= $date_start AND stamp <= $date_end";
} elseif ($timeperiod=="last_day") {
  $date_start = $date_end - (86400 * $time_x);
  $sql_date = "AND stamp >= $date_start AND stamp <= $date_end";
}

$avginterval=preg_replace('/[\D\.]/', '//', $avginterval);
if ($bytes != "" && $multiple != "" && trim($avginterval) != "" && $have_gd != "") {
// Make the multiple additive graph of all interfaces.
// We will NOT be allowing additive graphs of packets, drops or errors;
// it's kinda pointless, and this is enough bloat to add as it is. :)
// We can't allow an averaging interval of, like, one second... the script would never finish.
  if ($avginterval < 15) { $avginterval=15; }
  $ifidtext="(";
  $speed=array();
  foreach ($hostif as $host_if) {
// Split the $host_if variables into $host and $if.
    $tmp=preg_match('/([^!]+)!(.+)/', $host_if, $regs);
    $host=$regs[1];
    $if=$regs[2];
// Now we'll be needing the interface ID for these interfaces.
// Remember "localhost" entries aren't IN the "interfaces" table!
// And neither are interfaces that aren't being monitored anymore. Sigh.
    if ($host=="$local_hostname") {
      $sq="select a.id as id, b.speed as speed from stats_ifs a, localhost b " .
          "where (a.community='' AND a.hostname='$host' AND a.if_name='$if' AND " .
          "a.if_name=b.if_name)";
      $res=dosql($sq);
      if (mysql_num_rows($res) > 0) {
        $id = mysql_result($res, 0, "id");
        $speed[$id] = mysql_result($res, 0, "speed");
      } else { // Must be using SNMP even though it's from localhost...
        $sq="select a.id as id, b.speed as speed from stats_ifs a, interfaces b where " .
          "(a.hostname='$host' AND a.if_name='$if' AND a.hostname=b.hostname AND " .
          "a.if_name=b.if_name)";
        $res=dosql($sq);
        if (mysql_num_rows($res) > 0) {
          $id = mysql_result($res, 0, "id");
          $speed[$id] = mysql_result($res, 0, "speed");
        } else {
          $sq="select id from stats_ifs where hostname='$host' AND if_name='$if'";
          $res=dosql($sq);
          if (mysql_num_rows($res) > 0) {
            $id = mysql_result($res, 0, 0);
          }
          else {
            $id = -1;
          }
        }
      }
    } else { // Not from localhost.
      $sq="select a.id as id, b.speed as speed from stats_ifs a, interfaces b where " .
        "(a.hostname='$host' AND a.if_name='$if' AND a.hostname=b.hostname AND " .
        "a.if_name=b.if_name)";
      $res=dosql($sq);
      if (mysql_num_rows($res) > 0) {
        $id = mysql_result($res, 0, "id");
        $speed[$id] = mysql_result($res, 0, "speed");
      } else {
        $sq="select id from stats_ifs where hostname='$host' AND if_name='$if'";
        $res=dosql($sq);
        if (mysql_num_rows($res) > 0) {
          $id = mysql_result($res, 0, 0);
        }
        else {
          $id = -1;
        }
      }
    }
    $ifidtext .= "if_id=$id OR ";
  }
  $ifidtext = substr($ifidtext, 0, strlen($ifidtext)-4) . ")";
  $speedtotal=0;
  foreach ($speed as $s) { $speedtotal += $s; }

// Here's where we put together the pieces of our master query...
  $sq = "select ";
  if ($do_in) { $sq .= "r_bytes, "; }
  if ($do_out) { $sq .= "t_bytes, "; }
  $sq .= "if_id, stamp from stats where $ifidtext $sql_date order by if_id, stamp";

// Now we just slap our query into the stats table and see what we get.
// There have to be at least two rows returned in order to do any analysis,
// on pain of Divide By Zero, so we check for that in the results.
//echo "<h3>$sq</h3>\n";
  $res=dosql($sq);
  $numrows=mysql_num_rows($res);
  if ($numrows < 2) {
    echo "<BR>\nThere's not enough data in the database for this time period (".date($dateformat, $date_start)." to ".date($dateformat, $date_end).").  There must be at least two data points for analysis.\n</body></html>";
  } else {
    $cnt_stamps = $cnt_rbytes = $cnt_tbytes = array();
    $sqtmp = "select min(stamp) as smin, max(stamp) as smax from stats where $ifidtext $sql_date";
    $restmp=dosql($sqtmp);
    $lowstamp=mysql_result($restmp,0,"smin");
    $highstamp=mysql_result($restmp,0,"smax");
    $rb_prevslot=$tb_prevslot = $lowstamp;
    $rb_slot=$tb_slot = $lowstamp+$avginterval;
    $row2=mysql_fetch_array($res); // to initialize the previous stat entry.
    $total_interfaces_graphed=1;
// Start adding all the crap up. Note that the data will emerge from this pre-normalized!
// It will only need converting to averages once it's all summed up.
// This part *really* needs speeding up. It'd be a good place for a C module... hmmm.
// Just pass it the $res pointer, and bam...
    while ($row1=mysql_fetch_array($res)) {
      if ($row1["if_id"] == $row2["if_id"]) { // Still working on the same interface.
        if ($do_in) {
          if ($row1["stamp"] <= $rb_slot) {
// This entry is entirely in the current slot.
  // Add the whole intervening counter delta to this slot.
            $count=$row1["r_bytes"]-$row2["r_bytes"];
            if ($count < 0) { $count=$row1["r_bytes"]; }
            if ($cnt_rbytes[$rb_slot]=="") {
              $cnt_rbytes[$rb_slot]=0;
              if (! in_array($rb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$rb_slot;
              }
            }
            $cnt_rbytes[$rb_slot] += $count;
          } else {
// This entry is split across two or more slots.
  // First add in the partial counter delta for just the current slot...
            $pct=($rb_slot-$row2["stamp"]) / ($row1["stamp"]-$row2["stamp"]);
            $count=$row1["r_bytes"]-$row2["r_bytes"];
            if ($count < 0) { $count=$row1["r_bytes"]; }
            if ($cnt_rbytes[$rb_slot]=="") {
              $cnt_rbytes[$rb_slot]=0;
              if (! in_array($rb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$rb_slot;
              }
            }
            $cnt_rbytes[$rb_slot] += ($count * $pct);
  // Then add in the partial counter delta for each intervening WHOLE slot, IF ANY...
            $add=($avginterval/($row1["stamp"]-$row2["stamp"])) * $count;
            $rb_slot += $avginterval;
            $rb_prevslot += $avginterval;
            while ($row1["stamp"] > $rb_slot) {
              if ($cnt_rbytes[$rb_slot]=="") {
                $cnt_rbytes[$rb_slot]=0;
                if (! in_array($rb_slot, $cnt_stamps)) {
                  $cnt_stamps[]=$rb_slot;
                }
              }
              $cnt_rbytes[$rb_slot] += $add;
              $rb_slot += $avginterval;
              $rb_prevslot += $avginterval;
            }
  // And finally, add in the partial counter delta for the slot this stamp is actually in.
            $pct=($row1["stamp"]-$rb_prevslot) / ($row1["stamp"]-$row2["stamp"]);
            if ($cnt_rbytes[$rb_slot]=="") {
              $cnt_rbytes[$rb_slot]=0;
              if (! in_array($rb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$rb_slot;
              }
            }
            $cnt_rbytes[$rb_slot] += $count*$pct;
          }
        }
// Now do it all over again for the "outgoing" bytes.
// Doing it this way means lots more code, but it should be faster.
        if ($do_out) {
          if ($row1["stamp"] <= $tb_slot) {
            $count=$row1["t_bytes"]-$row2["t_bytes"];
            if ($count < 0) { $count=$row1["t_bytes"]; }
            if ($cnt_tbytes[$tb_slot]=="") {
              $cnt_tbytes[$tb_slot]=0;
              if (! in_array($tb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$tb_slot;
              }
            }
            $cnt_tbytes[$tb_slot] += $count;
          } else {
            $pct=($tb_slot-$row2["stamp"]) / ($row1["stamp"]-$row2["stamp"]);
            $count=$row1["t_bytes"]-$row2["t_bytes"];
            if ($count < 0) { $count=$row1["t_bytes"]; }
            if ($cnt_tbytes[$tb_slot]=="") {
              $cnt_tbytes[$tb_slot]=0;
              if (! in_array($tb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$tb_slot;
              }
            }
            $cnt_tbytes[$tb_slot] += ($count * $pct);
            $add=($avginterval/($row1["stamp"]-$row2["stamp"])) * $count;
            $tb_slot += $avginterval;
            $tb_prevslot += $avginterval;
            while ($row1["stamp"] > $tb_slot) {
              if ($cnt_tbytes[$tb_slot]=="") {
                $cnt_tbytes[$tb_slot]=0;
                if (! in_array($tb_slot, $cnt_stamps)) {
                  $cnt_stamps[]=$tb_slot;
                }
              }
              $cnt_tbytes[$tb_slot] += $add;
              $tb_slot += $avginterval;
              $tb_prevslot += $avginterval;
            }
            $pct=($row1["stamp"]-$tb_prevslot) / ($row1["stamp"]-$row2["stamp"]);
            if ($cnt_tbytes[$tb_slot]=="") {
              $cnt_tbytes[$tb_slot]=0;
              if (! in_array($tb_slot, $cnt_stamps)) {
                $cnt_stamps[]=$tb_slot;
              }
            }
            $cnt_tbytes[$tb_slot] += $count*$pct;
          }
        }
      } else {
// We'll be starting a new interface on the next $row1 fetch;
// the current $row1 values will become the "previous" $row2 value starting points.
        $rb_prevslot=$tb_prevslot=$lowstamp;
        $rb_slot=$tb_slot=$lowstamp+$avginterval;
        $total_interfaces_graphed++;
      }
      $row2=$row1;
    }
//    $speed=$speedtotal;
    sort($cnt_stamps);
    mysql_free_result($res);
    unset($row1, $row2, $res);

// Now convert the data to averages instead of running totals.
//   $test is just a bit more than the added ifSpeeds in bytes instead of bits...
    $test=($speedtotal/7);
    $prev_stamp=0;
    for ($i=0; $i<count($cnt_stamps); $i++) {
      $stamp=$cnt_stamps[$i];
      if ($do_in) {
        $cnt_rbytes[$stamp] /= ($stamp - $prev_stamp);
        if ($test > 0 && $cnt_rbytes[$stamp] >= $test) {
          $cnt_rbytes[$stamp]=$cnt_rbytes[$prev_stamp];
        }
      }
      if ($do_out) {
        $cnt_tbytes[$stamp] /= ($stamp - $prev_stamp);
        if ($test > 0 && $cnt_tbytes[$stamp] >= $test) {
          $cnt_tbytes[$stamp]=$cnt_tbytes[$prev_stamp];
        }
      }
      $prev_stamp=$stamp;
    }

// Display the low and high stamps from what we found so the user knows how it differs
// from their requested time frame, which it almost certainly will.
    echo "$HR\n<P><U>Sum of the $total_interfaces_graphed interfaces' averages " .
      "shown separately later on ($avginterval-sec. avg.):</U><BR>\nAnalyzed " .
      number_format($numrows) . " entries from " . date($dateformat, $cnt_stamps[0]) .
      " to " . date($dateformat, $cnt_stamps[count($cnt_stamps)-1]) . "<BR>\n";
    $int = $highstamp-$lowstamp;
    $out = longformat($int);
    echo "($out)<BR>\n";
//    echo "($lowstamp to $highstamp)<BR>\n";
//    if ($bitbyte=="B") { $speedtotal /= 8; }
    if (eregi("g", $g_bb)) {
      $bitbyte="B";
      $speedtotal /= 8;
    } else {
      $bitbyte="b";
    }
    echo "Total maximum speed of all interfaces: ".number_format($speedtotal)." $bitbyte/sec\n";

// Now... at long last... show the multi-graph.
    if ($have_gd != "" && eregi("g", $reportctl)) { // Necessary?
// Generate a nice random 16-digit number for naming the images.
      mt_srand((double)microtime()*1000000);
      $xtime  = (mt_rand(10000000, 99999999));
      $xtime .= (mt_rand(10000000, 99999999));
      if ($bitbyte=='b') {
        $reptype="bits";
      } else {
        $reptype="bytes";
      }
// Set the graphics format...
      switch($g_fmt) {
      case "j":
        $ext="jpg";
        break;
      case "p":
        $ext="png";
        break;
      default:
        $g_fmt="p";
        $ext="png";
        break;
      }
      if ($g_fmt=="j") {
        if (!imagejpeg(makegraph($cnt_rbytes, $cnt_tbytes, $reptype, 0), "bytes_$xtime.jpg", 85)) {
          echo "Couldn't create image file &quot;bytes_$xtime.jpg&quot;... probably write permissions.<BR>\n";
        }
      } elseif ($g_fmt=="p") {
       if (!imagepng(makegraph($cnt_rbytes, $cnt_tbytes, $reptype, 0), "bytes_$xtime.png")) {
          echo "Couldn't create image file &quot;bytes_$xtime.png&quot;... probably write permissions.<BR>\n";
        }
      }
      $tmp=preg_replace('/%%%MAPNAME%%%/', "bytes_$xtime", $map);
      echo "$tmp\n<P>$font2<B>Total average $reptype per second:</B></font><P>\n" .
           "<img src=\"bytes_$xtime.$ext\" border=0 ismap usemap=\"#bytes_$xtime\"></P>\n";
    }
    unset($cnt_stamps, $cnt_rbytes, $cnt_tbytes);
  }
}

// NOW, we show the non-multiple reports whether there was a multiple or not.
foreach ($hostif as $host_if) {
// Split the $host_if variables into $host and $if.
  $cir=0;
  $tmp=preg_match('/([^!]+)!(.*)/', $host_if, $regs);
  $host=$regs[1];
  $if=$regs[2];
  if (is_array($interfaces[$host])) {
    $hostalias=$interfaces[$host][0]["hostalias"];
    foreach($interfaces[$host] as $har) {
      if ($har["if_name"]==$if) {
        $cir=$har["cir"];
      }
    }
  }
  $alias="";
  $speed=0;
// Remember: $speed is BITS/SECOND, not bytes.  :-/
  if ($host==$local_hostname) {
    $alias=$local_aliases[$if];
    $speed=$local_speed[$if];
  }
  if ($alias=="" && is_array($interfaces[$host])) {
    foreach($interfaces[$host] as $tmp) {
      if ($tmp["if_name"]==$if) {
        $speed=$tmp["speed"];
        $alias=$tmp["alias"];
        break;
      }
    }
  }
  $test=($speed>0 ? $speed/7 : 0);
  if ($alias=="") {
    echo "$HR\n<P><U>Report for <B>&quot;$if&quot;</B> ";
  } else {
    echo "$HR\n<P><U>Report for <B>&quot;$alias&quot;</B> ($if) ";
  }
  if (trim($hostalias)=="") {
    echo "on <B>&quot;$host&quot;</B>:</U>\n";
  } else {
    echo "on <B>&quot;$hostalias&quot;</B> ($host):</U>\n";
  }
// Now we just slap it into the stats database and see what we get.
  $sqt="select id from stats_ifs where (hostname='$host' AND if_name='$if')";
  $rest=dosql($sqt);
  if (mysql_num_rows($rest) > 0) {
    $id = mysql_result($rest, 0, "id");
  } else {
    $id = -1;
  }
// We will ALWAYS be reading r_bytes, to see if the counters were reset.
  $sq = "select r_bytes, ";
  if ($bytes != "") {
    if ($do_out) {
      $sq .= "t_bytes, ";
    }
  }
  if ($packets != "") {
    if ($do_in) {
      $sq .= "r_packets, ";
    }
    if ($do_out) {
      $sq .= "t_packets, ";
    }
  }
  if ($errors != "") {
    if ($do_in) {
      $sq .= "r_errors, ";
    }
    if ($do_out) {
      $sq .= "t_errors, ";
    }
  }
  if ($dropped != "") {
    if ($do_in) {
      $sq .= "r_dropped, ";
    }
    if ($do_out) {
      $sq .= "t_dropped, ";
    }
  }
  $sq .= "stamp from stats where if_id=$id $sql_date order by stamp";
  $res=dosql($sq);
//echo "<h3>$sq</h3>\n";
  if (mysql_num_rows($res) < 2) {
    echo "<BR>\nThere's not enough data in the database for this time period (".date($dateformat, $date_start)." to ".date($dateformat, $date_end).").  There must be at least two data points for analysis.\n</body></html>";
    continue;
  }

  if ($have_gd != "" && eregi("g", $reportctl)) {
// Here are all the arrays for holding the various types of data from the database.
// Note that only the ones asked for on the form will actually be filled and
//   they're really more of a hash, using the timestamps themselves as the key.
    $cnt_rbytes=$cnt_rpackets=$cnt_rerrors=$cnt_rdropped=array();
    $cnt_tbytes=$cnt_tpackets=$cnt_terrors=$cnt_tdropped=array();
// This is the array that holds all the timestamps themselves, for later reference
//   when using the hashes above.  Note that no checking is done to make sure the same
//   stamp isn't used twice; if it is, it'll be in $cnt_stamps twice but both times it's
//   used it'll get the same value from the associated $cnt_ variable.
    $cnt_stamps=array();

// Here's my attempt to make up for missing entries in packets, drops, and errors during
// SNMP collection, where a machine reports something for Octets but one or more of the
// other types is missing. $rebooted is a flag that gets set if $prev_bytes *decreases*...
// meaning the machine's counters were reset. It should work in almost all cases, the
// only exception being the occasion when a machine resets and, when it comes back, its
// byte counters are somehow higher on the next collection cycle than before it rebooted.
// If that happens, well... who knows what can be done? :)
// And remember... $rebooted can also be set if they switch from 64-bit counters to 32-bit!
// There's no way to tell if that's happened...
    $rebooted=0;
// This *should* get set when an interface switches from 32- to 64-bit counters (avg>test).
    $switch64=0;

// Now we fill the arrays from the database, normalizing & averaging as we go...
    $row2=mysql_fetch_array($res);
    $first_stamp=$row2["stamp"];
    while ($row1=mysql_fetch_array($res)) {
      $cnt_stamps[]=$row1["stamp"];
      if (($row1["r_bytes"]-$row2["r_bytes"]) / ($row1["stamp"]-$row2["stamp"]) > $test) {
        $switch64 = 1;
      }
      if ($bytes != "") {
        if ($do_in) {
          $tmp=($row1["r_bytes"]-$row2["r_bytes"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($test>0 && $tmp > $test)) {
            $cnt_rbytes[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_rbytes[$row2["stamp"]] : 0);
          } else {
            $cnt_rbytes[$row1["stamp"]]=$tmp;
          }
        }
        if ($do_out) {
          $tmp=($row1["t_bytes"]-$row2["t_bytes"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($test>0 && $tmp > $test)) {
            $cnt_tbytes[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_tbytes[$row2["stamp"]] : 0);
          } else {
            $cnt_tbytes[$row1["stamp"]]=$tmp;
          }
        }
      }
      if ($packets != "") {
        if ($do_in) {
          $tmp=($row1["r_packets"]-$row2["r_packets"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($switch64==1 || $tmp < 0 || ($rebooted == 0 && $row2["r_packets"] == 0)) {
            $cnt_rpackets[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_rpackets[$row2["stamp"]] : 0);
          } else {
            $cnt_rpackets[$row1["stamp"]]=$tmp;
          }
        }
        if ($do_out) {
          $tmp=($row1["t_packets"]-$row2["t_packets"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($switch64==1 || $tmp < 0 || ($rebooted == 0 && $row2["t_packets"] == 0)) {
            $cnt_tpackets[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_tpackets[$row2["stamp"]] : 0);
          } else {
            $cnt_tpackets[$row1["stamp"]]=$tmp;
          }
        }
      }
      if ($errors != "") {
        if ($do_in) {
          $tmp=($row1["r_errors"]-$row2["r_errors"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($rebooted == 0 && $row2["r_errors"] == 0)) {
            $cnt_rerrors[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_rerrors[$row2["stamp"]] : 0);
          } else {
            $cnt_rerrors[$row1["stamp"]]=$tmp;
          }
        }
        if ($do_out) {
          $tmp=($row1["t_errors"]-$row2["t_errors"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($rebooted == 0 && $row2["t_errors"] == 0)) {
            $cnt_terrors[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_terrors[$row2["stamp"]] : 0);
          } else {
            $cnt_terrors[$row1["stamp"]]=$tmp;
          }
        }
      }
      if ($dropped != "") {
        if ($do_in) {
          $tmp=($row1["r_dropped"]-$row2["r_dropped"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($rebooted == 0 && $row2["r_dropped"] == 0)) {
            $cnt_rdropped[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_rdropped[$row2["stamp"]] : 0);
          } else {
            $cnt_rdropped[$row1["stamp"]]=$tmp;
          }
        }
        if ($do_out) {
          $tmp=($row1["t_dropped"]-$row2["t_dropped"]) / ($row1["stamp"]-$row2["stamp"]);
          if ($tmp < 0 || ($rebooted == 0 && $row2["t_dropped"] == 0)) {
            $cnt_tdropped[$row1["stamp"]]=(count($cnt_stamps) > 1 ? $cnt_tdropped[$row2["stamp"]] : 0);
          } else {
            $cnt_tdropped[$row1["stamp"]]=$tmp;
          }
        }
      }
      $switch64=0;
      if ($row1["r_bytes"] < $row2["r_bytes"]) {
        $rebooted=1;
      } else {
        $rebooted=0;
      }
      $row2=$row1;
    }

// ...and display the low and high stamps from what we found so the user knows how it differs
// from their requested time frame, which it almost certainly will.
    echo "<BR>\nAnalyzed " . number_format(count($cnt_stamps)+1) . " entries from " .
      date($dateformat, $first_stamp) . " to " .
      date($dateformat, $cnt_stamps[count($cnt_stamps)-1]) . "<BR>\n";

    $low = $first_stamp;
    $high = $cnt_stamps[count($cnt_stamps)-1];
    $int = $high-$low;
    $out = longformat($int);
    echo "($out)<BR>\n";
// echo "($first_stamp to {$cnt_stamps[count($cnt_stamps)-1]})<BR>\n";
//    if ($bitbyte=="B") { $speeddiv=8; } else { $speeddiv=1; }

    if (eregi("g", $g_bb)) {
      $bitbyte="B";
      $speeddiv=8;
    } else {
      $bitbyte="b";
      $speeddiv=1;
    }

    echo "Maximum speed of this interface: ";
    if ($speed==0) {
      echo "<I>Unknown</I><BR>\n";
    } else {
      echo number_format($speed/$speeddiv)." $bitbyte/sec<BR>\n";
    }

// Now... show the graphs.
// Generate a nice random 16-digit number for each host to use in naming its JPGs.
    mt_srand((double)microtime()*1000000);
    $xtime  = (mt_rand(10000000, 99999999));
    $xtime .= (mt_rand(10000000, 99999999));
    if ($bitbyte=='B') {
      $reptype="bytes";
    } else {
      $reptype="bits";
    }
    switch($g_fmt) {
    case "j":
      $ext="jpg";
      break;
    case "p":
      $ext="png";
      break;
    default:
      $g_fmt="j";
      $ext="jpg";
    }
    if ($alias=="") {
      $ifal=$if;
    } else {
      $ifal=$alias;
    }
    if (trim($hostalias)=="") {
      $hostal=$host;
    } else {
      $hostal=$hostalias;
    }
    if ($bytes != "") {
      if ($g_fmt=="j") {
        if (!imagejpeg(makegraph($cnt_rbytes, $cnt_tbytes, $reptype, $cir), "bytes_$xtime.jpg", 85)) {
          echo "Couldn't create image file &quot;bytes_$xtime.jpg&quot;... probably write permissions.<BR>\n";
        }
      } elseif ($g_fmt=="p") {
        if (!imagepng(makegraph($cnt_rbytes, $cnt_tbytes, $reptype, $cir), "bytes_$xtime.png")) {
          echo "Couldn't create image file &quot;bytes_$xtime.png&quot;... probably write permissions.<BR>\n";
        }
      }
      $tmp=preg_replace('/%%%MAPNAME%%%/', "bytes_$xtime", $map);
      echo "$tmp\n<P>$font2<B>Average $reptype per second transferred on $hostal-&gt;$ifal:</B></font><P>\n" .
           "<img src=\"bytes_$xtime.$ext\" border=0 ismap usemap=\"#bytes_$xtime\"></P>\n";

    }
    if ($packets != "") {
      if ($g_fmt=="j") {
        if (!imagejpeg(makegraph($cnt_rpackets, $cnt_tpackets, "packets", 0), "packets_$xtime.jpg", 85)) {
          echo "Couldn't create image file &quot;packets_$xtime.jpg&quot;... probably write permissions.<BR>\n";
        }
      } elseif ($g_fmt=="p") {
        if (!imagepng(makegraph($cnt_rpackets, $cnt_tpackets, "packets", 0), "packets_$xtime.png")) {
          echo "Couldn't create image file &quot;packets_$xtime.png&quot;... probably write permissions.<BR>\n";
        }
      }
      $tmp=preg_replace('/%%%MAPNAME%%%/', "packets_$xtime", $map);
      echo "<P>$font2<B>Average packets per second transferred on $hostal-&gt;$ifal:</B></font><P>\n" .
           "<img src=\"packets_$xtime.$ext\" border=0 ismap usemap=\"#packets_$xtime\"></P>\n";
    }
    if ($errors != "") {
      if ($g_fmt=="j") {
        if (!imagejpeg(makegraph($cnt_rerrors, $cnt_terrors, "errors", 0), "errors_$xtime.jpg", 85)) {
          echo "Couldn't create image file &quot;errors_$xtime.jpg&quot;... probably write permissions.<BR>\n";
        }
      } elseif ($g_fmt=="p") {
        if (!imagepng(makegraph($cnt_rerrors, $cnt_terrors, "errors", 0), "errors_$xtime.png")) {
          echo "Couldn't create image file &quot;errors_$xtime.png&quot;... probably write permissions.<BR>\n";
        }
      }
      $tmp=preg_replace('/%%%MAPNAME%%%/', "errors_$xtime", $map);
      echo "<P>$font2<B>Average errors per second on $hostal-&gt;$ifal:</B></font><P>\n" .
           "<img src=\"errors_$xtime.$ext\" border=0 ismap usemap=\"#errors_$xtime\"></P>\n";
    }
    if ($dropped != "") {
      if ($g_fmt=="j") {
        if (!imagejpeg(makegraph($cnt_rdropped, $cnt_tdropped, "dropped", 0), "dropped_$xtime.jpg", 85)) {
          echo "Couldn't create image file &quot;dropped_$xtime.jpg&quot;... probably write permissions.<BR>\n";
        }
      } elseif ($g_fmt=="p") {
        if (!imagepng(makegraph($cnt_rdropped, $cnt_tdropped, "dropped", 0), "dropped_$xtime.png")) {
          echo "Couldn't create image file &quot;dropped_$xtime.png&quot;... probably write permissions.<BR>\n";
        }
      }
      $tmp=preg_replace('/%%%MAPNAME%%%/', "dropped_$xtime", $map);
      echo "<P>$font2<B>Average dropped Packets per second on $hostal-&gt;$ifal:</B></font><P>\n" .
           "<img src=\"dropped_$xtime.$ext\" border=0 ismap usemap=\"#dropped_$xtime\"></P>\n";
    }
  }

  if (eregi("r", $reportctl)) {
// Make the text-only report tables.
// First we need to (again) fill the arrays from the database.
// It is indeed unfortunate that we have to recreate all the cnt_* arrays,
// but rewriting the graphing thingy above makes it necessary... :(
// At least we can reuse the same result set... IF they ALSO got graphs.
// If not, this will be the first place it gets created or referenced!
    if (! isset($res)) {
      $res=dosql($sq);
    } else {
      mysql_data_seek($res,0);
    }
    $cnt_stamps=$cnt_rbytes=$cnt_tbytes=$cnt_rpackets=$cnt_tpackets=$cnt_rerrors=$cnt_terrors=$cnt_rdropped=$cnt_tdropped=array();
    while ($row=mysql_fetch_array($res)) {
      $stamp=$row["stamp"];
      $cnt_stamps[]=$stamp;
      if ($bytes != "") {   
        if ($do_in) {
          $cnt_rbytes[$stamp] = $row["r_bytes"];
        }
        if ($do_out) {
          $cnt_tbytes[$stamp] = $row["t_bytes"];
        }
      }
      if ($packets != "") { 
        if ($do_in) { 
          $cnt_rpackets[$stamp] = $row["r_packets"];
        }
        if ($do_out) {
          $cnt_tpackets[$stamp] = $row["t_packets"];   
        }
      }
      if ($errors != "") {
        if ($do_in) {
          $cnt_rerrors[$stamp] = $row["r_errors"];
        }
        if ($do_out) { 
          $cnt_terrors[$stamp] = $row["t_errors"];
        }
      }
      if ($dropped != "") {
        if ($do_in) {
          $cnt_rdropped[$stamp] = $row["r_dropped"];
        }
        if ($do_out) {   
          $cnt_tdropped[$stamp] = $row["t_dropped"];
        }
      }
    }
    mysql_free_result($res);
// Now. Normalize the data so it isn't running-totals anymore...
    if ($bytes != "") {
      if ($do_in) {
        $cnt_rbytes=normalize($cnt_rbytes);
      }
      if ($do_out) {
        $cnt_tbytes=normalize($cnt_tbytes);
      }
// Now run it through 95th-percentile processing if required.
      if ($do95) { do95($cnt_rbytes, $cnt_tbytes); }
    }
    if ($packets != "") {   
      if ($do_in) {
        $cnt_rpackets=normalize($cnt_rpackets);
      }
      if ($do_out) {
        $cnt_tpackets=normalize($cnt_tpackets);
      }
    }
    if ($errors != "") {
      if ($do_in) {
        $cnt_rerrors=normalize($cnt_rerrors);
      }
      if ($do_out) {
        $cnt_terrors=normalize($cnt_terrors);
      }
    }
    if ($dropped != "") {
      if ($do_in) {
        $cnt_rdropped=normalize($cnt_rdropped);
      }
      if ($do_out) {
        $cnt_tdropped=normalize($cnt_tdropped);
      }
    }

// $colspan is self-explanatory, but I'll explain anyway... For each of the possibly-asked-for
// report categories, add more columns to the cells that span one or more columns...
// and remember $colspan gets used twice per row; once for incoming, once for outgoing.
    $colspan=0;
    if ($bytes != "") { $colspan+=2; }
    if ($packets != "") { $colspan+=2; }
    if ($errors != "") { $colspan++; }
    if ($dropped != "") { $colspan++; }
    echo "<P><center>\n";
    $z=$colspan*2+2;
    $rpttext = "<table border=1>\n" .
      "<tr><td colspan=$z>Lines with <font color=#CC0000>red times</font> in the " .
      "following report indicate that line contains a gap in the collected data. " .
      "This script only detects gaps larger than your specified summary interval (" .
      number_format($summ_interval) . " seconds). Click a \"From\" time to zoom in there.</td></tr>\n" .
      "<tr><td rowspan=2 align=center>" .
      "<B>From:</B></td>\n<td rowspan=2 align=center><B>To:</B></td>\n" .
      "<td colspan=$colspan align=center><B>Incoming</B></td>" .
      "<td colspan=$colspan align=center><B>Outgoing</B></td></tr>\n";

// Output the header row.
    $tmp="";
    if ($bytes != "") {
      $tmp .= "<td align=center>";
//      if ($bitbyte=="b") { $tmp .= "Bits"; } else { $tmp .= "Bytes"; }
      if (eregi("r", $g_bb)) {
        $tmp .= "Bytes";
        $bitbyte="B";
      } else {
        $tmp .= "Bits";
        $bitbyte="b";
      }
      $tmp .= "</td>\n<td align=center>Avg $bitbyte/s</td>\n";
    }
    if ($packets != "") { $tmp .= "<td align=center>Packets</td>\n<td align=center>Avg p/s</td>\n"; }
    if ($errors != "") { $tmp .= "<td align=center>Errs</td>\n"; }
    if ($dropped != "") { $tmp .= "<td align=center>Drops</td>\n"; }
    $rpttext .= "<tr>$tmp$tmp</tr>\n";

// Set some initial values... counters and registers and such; you know, geeky stuff. :)
// $tot_ vars are the running totals which are cleared after being shown on each report row.
// $grand_ vars are the grand totals which are displayed as the very last report row.
// $cnt_ vars are of course the normalized values from the database.

    $from = $cnt_stamps[0];
    $to = $from + $summ_interval;
    $tot_rbytes=$tot_tbytes=$grand_rbytes=$grand_tbytes=0;
    $tot_rpackets=$tot_tpackets=$grand_rpackets=$grand_tpackets=0;
    $tot_rerrors=$tot_terrors=$grand_rerrors=$grand_terrors=0;
    $tot_rdropped=$tot_tdropped=$grand_rdropped=$grand_tdropped=0;
    $gaptext="<table border=1>\n<tr><td colspan=2 align=center>" .
      "<B>Detected Data Gaps</B></td></tr>\n" .
      "<tr><td align=center><B>From:</B></td>\n" .
      "<td align=center><B>To:</B></td></tr>\n";
    $gaptest=strlen($gaptext);
//    if ($bitbyte=="b") { $speedmod=8; } else { $speedmod=1; }
    if (eregi("r", $g_bb)) {
      $speedmod=1;
      $bitbyte="B";
    } else {
      $speedmod=8;
      $bitbyte="b";
    }
    $speedcheck=$speed/4; // Two times the speed, converted to bytes/sec.

// And now, here we go creating the data to show on the table.
// Fortunately, only r/t_bytes and _packets are affected by an SNMP version switch.
    $flubber["summ_interval"]=ceil($summ_interval/10);
    $flubber["summ_units"]="second";
    for ($i=0; $i<count($cnt_stamps); $i++) {
      $stamp=$cnt_stamps[$i];
      if ($bytes != "") {
        if ($speedcheck==0 || ($cnt_rbytes[$stamp] - $cnt_rbytes[$cnt_stamps[$i-1]]) /
                              ($stamp - $cnt_stamps[$i-1]) <= $speedcheck) {
          $tot_rbytes += ($cnt_rbytes[$stamp] * $speedmod);
          $tot_tbytes += ($cnt_tbytes[$stamp] * $speedmod);
        }
      }
      if ($packets != "") {
        if ($speedcheck==0 || ($cnt_rbytes[$stamp] - $cnt_rbytes[$cnt_stamps[$i-1]]) /
                              ($stamp - $cnt_stamps[$i-1]) <= $speedcheck) {
          $tot_rpackets += $cnt_rpackets[$stamp];
          $tot_tpackets += $cnt_tpackets[$stamp];
        }
      }
      if ($errors != "") {
        $tot_rerrors += $cnt_rerrors[$stamp];
        $tot_terrors += $cnt_terrors[$stamp];
      }
      if ($dropped != "") {
        $tot_rdropped += $cnt_rdropped[$stamp];
        $tot_tdropped += $cnt_tdropped[$stamp];
      }
      if ($stamp >= $to || $i == (count($cnt_stamps) - 1)) {
        if ($stamp > ($cnt_stamps[$i-1] + ($summ_interval * 2))) { // there must be a gap...
          $fontz="<font color=#CC0000>";
          $gaptext .= "<tr><td>" . date($dateformat, $cnt_stamps[$i-1]) .
          "</td><td>" . date($dateformat, $stamp) . "</td></tr>\n";
        } else {
          $fontz="";
        }
        $to = $stamp;
        $int = $to - $from;
        $tmpi = $tmpo = "";
        if ($bytes != "") {
          $tmpi .= "<td align=right class=\"small\">".number_format($tot_rbytes)."</td>\n" .
            "<td align=right class=\"small\">".number_format(sprintf("%0.2f", $tot_rbytes/$int), 2)."</td>\n";
          $tmpo .= "<td align=right class=\"small\">".number_format($tot_tbytes)."</td>\n" .
            "<td align=right class=\"small\">".number_format(sprintf("%0.2f", $tot_tbytes/$int), 2)."</td>\n";
        }
        if ($packets != "") {
          $tmpi .= "<td align=right class=\"small\">".number_format($tot_rpackets)."</td>\n" .
            "<td align=right class=\"small\">".number_format(sprintf("%0.2f", $tot_rpackets/$int), 2)."</td>\n";
          $tmpo .= "<td align=right class=\"small\">".number_format($tot_tpackets)."</td>\n" .
            "<td align=right class=\"small\">".number_format(sprintf("%0.2f", $tot_tpackets/$int), 2)."</td>\n";
        }
        if ($errors != "") {
          $tmpi .= "<td align=right class=\"small\">".number_format($tot_rerrors)."</td>\n";
          $tmpo .= "<td align=right class=\"small\">".number_format($tot_terrors)."</td>\n";
        }
        if ($dropped != "") {
          $tmpi .= "<td align=right class=\"small\">".number_format($tot_rdropped)."</td>\n";
          $tmpo .= "<td align=right class=\"small\">".number_format($tot_tdropped)."</td>\n";
        }
        list($flubber["f_month"], $flubber["f_day"], $flubber["f_year"], $flubber["f_hour"],
             $flubber["f_min"], $flubber["f_sec"])=preg_split('/:/', date("n:j:Y:G:i:s", $from));
        list($flubber["t_month"], $flubber["t_day"], $flubber["t_year"], $flubber["t_hour"],
             $flubber["t_min"], $flubber["t_sec"])=preg_split('/:/', date("n:j:Y:G:i:s", $to));
        $zoomurl="<a href=\"$PHP_SELF?flubber=".rawurlencode(serialize($flubber))."\">";
        $rpttext .= "<tr><td class=\"small\">$zoomurl$fontz".date($dateformat, $from)."</a></td>\n" .
          "<td class=\"small\">$fontz".date($dateformat, $to)."</td>$tmpi$tmpo</tr>\n";
        $from = $cnt_stamps[$i];
        $to = $from + $summ_interval;
        $grand_rbytes += $tot_rbytes;
        $grand_tbytes += $tot_tbytes;
        $grand_rpackets += $tot_rpackets;
        $grand_tpackets += $tot_tpackets;
        $grand_rerrors += $tot_rerrors;
        $grand_terrors += $tot_terrors;
        $grand_rdropped += $tot_rdropped;
        $grand_tdropped += $tot_tdropped;
        $tot_rbytes=$tot_rpackets=$tot_rerrors=$tot_rdropped=0;
        $tot_tbytes=$tot_tpackets=$tot_terrors=$tot_tdropped=0;
      }
    }

    $low = $cnt_stamps[0];
    $high = $cnt_stamps[count($cnt_stamps)-1];
    $int = $high-$low;
    $out = longformat($int);
    $rpttext .= "<tr><td colspan=2 rowspan=3 align=center class=\"small\"><b><big>Grand Totals</big></b><BR>" .
      number_format(sprintf("%0.2f",(($grand_rbytes+$grand_tbytes)/1000000)),2) . " M$bitbyte<br>\n" .
      "($out)</td>\n<td colspan=$colspan align=center><B>Incoming</B></td>" .
      "<td colspan=$colspan align=center><B>Outgoing</B></td></tr>\n";
    $tmp="";
    if ($bytes != "") { $tmp .= "<td align=center>";
      if ($bitbyte=="b") { $tmp .= "Bits"; } else { $tmp .= "Bytes"; }
      $tmp .= "</td>\n<td align=center>Avg $bitbyte/s</td>\n"; 
    }
    if ($packets != "") { $tmp .= "<td align=center>Packets</td>\n<td align=center>Avg p/s</td>\n"; }
    if ($errors != "") { $tmp .= "<td align=center>Errs</td>\n"; }
    if ($dropped != "") { $tmp .= "<td align=center>Drops</td>\n"; }
    $rpttext .= "<tr>$tmp$tmp</tr>\n";

    $tmpi = $tmpo = "";
    if ($bytes != "") {
      $tmpi .= "<td align=right class=\"small\">" . number_format($grand_rbytes) . 
        "</td>\n<td align=right class=\"small\">" . number_format(sprintf("%0.2f", $grand_rbytes/$int), 2) .
        "</td>\n";
      $tmpo .= "<td align=right class=\"small\">" . number_format($grand_tbytes) .
        "</td>\n<td align=right class=\"small\">" . number_format(sprintf("%0.2f", $grand_tbytes/$int), 2) .
        "</td>\n";
    }
    if ($packets != "") {
      $tmpi .= "<td align=right class=\"small\">" . number_format($grand_rpackets) .
        "</td>\n<td align=right class=\"small\">" . number_format(sprintf("%0.2f", $grand_rpackets/$int), 2) .
        "</td>\n";
      $tmpo .= "<td align=right class=\"small\">" . number_format($grand_tpackets) .
        "</td>\n<td align=right class=\"small\">" . number_format(sprintf("%0.2f", $grand_tpackets/$int), 2) .
        "</td>\n";
    }
    if ($errors != "") {
      $tmpi .= "<td align=right class=\"small\">" . number_format($grand_rerrors) . "</td>\n";
      $tmpo .= "<td align=right class=\"small\">" . number_format($grand_terrors) . "</td>\n";
    }
    if ($dropped != "") {
      $tmpi .= "<td align=right class=\"small\">" . number_format($grand_rdropped) . "</td>\n";
      $tmpo .= "<td align=right class=\"small\">" . number_format($grand_tdropped) . "</td>\n";
    }
    $rpttext .= "<tr>$tmpi$tmpo</tr>\n</table>\n";
    if ($gaptest == strlen($gaptext)) {
      $gaptext="";
    } else {
      $gaptext .= "</table>\n";
    }
    echo "$gaptext$rpttext</center>\n";
  } else {
    mysql_free_result($res);
  }
}

// Now let's clear out the incredible volumes of memory we used...
// nope, doesn't quench httpd memory usage. I've leave it anyway.
unset($cnt_rbytes,$cnt_rpackets,$cnt_rerrors,$cnt_rdropped,
      $cnt_tbytes,$cnt_tpackets,$cnt_terrors,$cnt_tdropped,
      $cnt_stamps, $res);

$out=formattime(getmicrotime() - $start_stopwatch);
$random_msgs .= "Report generation took $out.<BR>\n";

if ($random_msgs != "") {
  echo "$HR\n<P><font color=#$graph_text>$random_msgs</font></P>\n";
  $random_msgs="";
}

echo "<P>\n<a href=\"index.php\">Back To The Menu</a>\n</body></html>";
die; 
?>
